@@ -1,8 +1,6 @@
-context("similarConfigurations")
-
 test_that("similarConfigurations", {
 
-  parameters <- irace:::readParameters(text = '
+  parameters <- readParameters(text = '
 n1 "" r (0,1)
 n2 "" r (0,1)
 n3 "" r (0,1)
